require 'crowd_fund/fund_request'
require 'crowd_fund/die'
require 'crowd_fund/funding_round'

module CrowdFund
  describe FundRequest do
    before do
      $stdout = StringIO.new
      @funding_scenario = FundRequest.new("test_funding")

      @amount = 5000
      @target = 10000
      @amount_to_add = 500
      @amount_to_remove = 125
      @project = Project.new("new_project", @amount, @target)

      @funding_scenario.add_project(@project)
    end

    it "formats the entries of to_hit_target projects" do
      string = @funding_scenario.to_hit_target_entry(@project)

      expect(string).to eq("#{@project.name.ljust(50, '.')} (#{@project.total_to_fund} to hit target)")
    end

    it "gives a random pledge to a project during a funding turn" do
      @funding_scenario.request_funding(1)

      expect(@project.total_pledges).not_to be_zero
    end

    it "adds funds and pledges when a lower number is rolled" do
      allow_any_instance_of(Die).to receive(:roll).and_return(1)

      @funding_scenario.request_funding(1)


      expect(@project.amount).to eq(@amount + @amount_to_add + @project.total_pledges)
    end

    it "removes funds when a higher number is rolled" do
      allow_any_instance_of(Die).to receive(:roll).and_return(5)

      @funding_scenario.request_funding(1)

      expect(@project.amount).to eq(@amount + @project.total_pledges - @amount_to_remove)
    end

    it "skips funding when a medium number is rolled" do
      allow_any_instance_of(Die).to receive(:roll).and_return(3)

      @funding_scenario.request_funding(1)

      expect(@project.amount).to eq(@amount + @project.total_pledges)
    end

    context "at the end of funding scenario" do

      before do
        @project = Project.new("new_project", @amount, @target)
        @project2 = Project.new("foo_project", 100, 1000)

        @funding_scenario.add_project(@project)
        @funding_scenario.add_project(@project2)

        @project.receive_pledge(Pledge.new(:bronze, 50))
        @project2.receive_pledge(Pledge.new(:gold, 100))
        @project.receive_pledge(Pledge.new(:silver, 75))
        @project2.receive_pledge(Pledge.new(:bronze, 50))
      end

      it "computes the total pledges given to all projects" do
        expect(@funding_scenario.total_pledges_given).to eq(275)
      end
    end
  end
end
